==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/export.cc <==
#include <string.h>
#include <stdlib.h>
#include <misc.h>
#include "projet.h"
#include "projetx.m"

/*
	Vrifie si un livraison est prsente dans un path.
	Retourne != 0 si oui.
*/
static int export_check (
	char *path)
{
	char zip[MAXSIZ_PATH];
	file_chgext (path,zip,"zip");
	return file_type(zip)==0 || file_type(path)==1;
}

/*
	Localise le rpertoire (ou fichier .zip) d'une livraison.
	Retourne -1 si trouve pas.
*/
static int export_locate(
	USERINFO *user,
	const char *revpath,	// Rvision en format path _x_y_z_2
	char *path)
{
	/* #Spcification: admin / export / provenance des fichiers
		Les fichiers requis pour l'exportation (une rvision)
		proviennent soit de /kit/livre ou d'un des /kit/sync/site.

		Ceux de /kit/livre proviennent des livraisons locales.
		Ceux de /kit/sync/site proviennent des livraisons recus
		de ces sites.

		De plus, ces livraisons peuvent tre en format .zip ou
		tale dans un sous-rpertoire. La commande envoie
		devra traiter les deux cas.
	*/
	user->maklivpath (revpath,path);
	int ret = 0;
	if (export_check(path)){
		ret = 1;
	}else{
		// Il faut chercher dans /kit/sync
		/* #Spcification: admin / export / limite
			Le nombre de site dans /kit/sync est limit arbitrairement
			 100.
		*/
		char *tb[100];

		/* #Spcification: admin / export / /kit/sync
			Lorsqu'on recoie une rvision d'un site, on la place
			dans le sous-rpertoire de /kit/sync appropri. L'importation
			(Option admin/import de projetx) pourra alors import la rvision
			et dtruire le fichier. projetx pourra alors rviser le contenu
			de /kit/sync/XX/version.dat.

			L'exportation consulte aussi les diffrents rpertoire
			/kit/sync/sites pour retrouver les livraisons.
		*/
		int nb = dir_getlistd (user->getsyncpath(),WILD_ALLFILE,0,tb,100);
		for (int i=0; i<nb; i++){
			path_make (tb[i],revpath,path);
			if (export_check(path)){
				ret = 1;
				break;
			}
		}
		tbstr_free (tb,nb);
	}
	return ret;
}

/*
	Dtermine toutes les rvisions qui doivent tre export vers un site
	et dclenche le traitement pour raliser.

	Retourne -1 si erreur.
*/
int export_site (
	USERINFO *user,
	VERSION_DAT &princ,
	const char *site,
	PROJET_LOG &log)
{
	/* #Spcification: admin / export / /kit/sync

		/kit/livre opre avec un autre systme, soit /kit/sync. /kit/sync
		contient un rpertoire pour chaque site qui doit tre mis
		 jour. Dans chaque sous-rpertoire de /kit/sync, il y a
		au moins deux fichiers et parfois quelques .zip.
	*/
	int ret = 0;
	/* #Spcification: admin / export / /kit/sync
		Le fichier /kit/sync/XX/version.dat est une copie du fichier
		/kit/ombre/version.dat du site XX. En calculant la diffrence
		entre ce fichier et la version locale de ce fichier
		(/kit/ombre/version.dat), on peut tablir la liste des
		rvisions qui doivent tre transmises au site XX.
		Ces rvisions sont dans /kit/livre et aussi dans les autres
		rpertoires /kit/sync/sites.
	*/
	char path[MAXSIZ_PATH];
	user->maksyncpath (site,path);
	char vpath[MAXSIZ_PATH];
	path_make (path,"version.dat",vpath);
	VERSION_DAT exp (vpath);
	int save_exp = 0;
	int nb=princ.getnb();
	for (int i=0; i<nb; i++){
		VERSION_ONE *one = princ.get(i);
		if (one->getprog()!=NULL){
			int no;
			const REVISION *rev = one->getrev();
			VERSION_ONE *sone = exp.get(rev,no);
			if (sone == NULL){
				// Il faut exporter cette rvision au site
				char revstr[MAXSIZ_REVISION];
				rev->format (revstr);
				char revpath[MAXSIZ_NAME];
				rev->formatpath(revpath);
				char basedir[MAXSIZ_PATH];
				if (export_locate (user,revpath,basedir)){
					/* #Spcification: admin / export / script
						Le fichier /kit/sync/XX/envoie est script UNIX qui est
						activ par le programme prjexpr et transmet chaque rvision
						selon le mcanisme appropri pour un site.

						Il recoit comme seul argument le path absolue (sans
						extension) du repertoire ou du fichier .zip qui
						contient la livraison.
					*/
					char cmd[MAXSIZ_PATH];
					path_make (path,"envoie",cmd);
					path_2os (cmd,cmd);
					strcat (cmd," ");
					path_2os (basedir,basedir);
					strcat (cmd,basedir);
					strcat (cmd," ");
					strcat (cmd,revpath);
					if (system(cmd)==0){
						log.printf ("exporte %s\n",revstr);
						VERSION_ONE *newone = new VERSION_ONE (one);
						if (newone != NULL){
							exp.add (newone);
							save_exp = 1;
						}
					}else{
						log.printf ("**** Erreur durant l'exportation de %s\n"
							,revstr);
						xconf_error (MSG_U(E_ERREXP
							,"Erreur durant l'exportation\n"
							 "de la rvision %s")
							,revstr);
						ret = -1;
					}
				}else{
					log.printf ("**** Pas de livraison disponible pour"
						" exporter %s\n",revstr);
					xconf_error (MSG_U(E_MISSEXP
						,"Pas de livraison disponible\n"
						 "pour exporter la rvision %s")
						,revstr);
					ret = -1;
				}
			}
		}
	}
	if (save_exp) exp.save(user);
	return ret;
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/export.cc <==
#include <string.h>
#include <stdlib.h>
#include <misc.h>
#include <popen.h>
#include "projet.h"
#include "projetx.m"

/*
	Vrifie si un livraison est prsente dans un path.
	Retourne != 0 si oui.
*/
static int export_check (
	char *path)
{
	char zip[MAXSIZ_PATH],tar[MAXSIZ_PATH];
	file_chgext (path,zip,"zip");
	file_chgext (path,tar,"tar.gz");
	return file_type(tar)==0 || file_type(zip)==0 || file_type(path)==1;
}

/*
	Localise le rpertoire (ou fichier .zip) d'une livraison.
	Retourne -1 si trouve pas.
*/
static int export_locate(
	USERINFO *user,
	const char *revpath,	// Rvision en format path _x_y_z_2
	char *path)
{
	/* #Spcification: admin / export / provenance des fichiers
		Les fichiers requis pour l'exportation (une rvision)
		proviennent soit de /kit/livre ou d'un des /kit/sync/site.

		Ceux de /kit/livre proviennent des livraisons locales.
		Ceux de /kit/sync/site proviennent des livraisons recus
		de ces sites.

		De plus, ces livraisons peuvent tre en format .zip ou
		tale dans un sous-rpertoire. La commande envoie
		devra traiter les deux cas.
	*/
	user->maklivpath (revpath,path);
	int ret = 0;
	if (export_check(path)){
		ret = 1;
	}else{
		// Il faut chercher dans /kit/sync
		/* #Spcification: admin / export / limite
			Le nombre de site dans /kit/sync est limit arbitrairement
			 100.
		*/
		char *tb[100];

		/* #Spcification: admin / export / /kit/sync
			Lorsqu'on recoie une rvision d'un site, on la place
			dans le sous-rpertoire de /kit/sync appropri. L'importation
			(Option admin/import de projetx) pourra alors import la rvision
			et dtruire le fichier. projetx pourra alors rviser le contenu
			de /kit/sync/XX/version.dat.

			L'exportation consulte aussi les diffrents rpertoire
			/kit/sync/sites pour retrouver les livraisons.
		*/
		int nb = dir_getlistd (user->getsyncpath(),WILD_ALLFILE,0,tb,100);
		for (int i=0; i<nb; i++){
			path_make (tb[i],revpath,path);
			if (export_check(path)){
				ret = 1;
				break;
			}
		}
		tbstr_free (tb,nb);
	}
	return ret;
}

static int prjexpr_system (const char *cmd)
{
	int ret = -1;
	POPEN pop (cmd);
	if (pop.isok()){
		while (pop.wait(1000)>=0){
			char buf[1000];
			while (pop.readout(buf,sizeof(buf)-1)!=-1){
				fputs (buf,stdout);
			}
			while (pop.readerr(buf,sizeof(buf)-1)!=-1){
				fputs (buf,stderr);
			}

		}
		ret = pop.getstatus();
	}
	return ret;
}

/*
	Dtermine toutes les rvisions qui doivent tre export vers un site
	et dclenche le traitement pour raliser.

	Retourne -1 si erreur.
*/
int export_site (
	USERINFO *user,
	VERSION_DAT &princ,
	const char *site,
	PROJET_LOG &log)
{
	/* #Spcification: admin / export / /kit/sync

		/kit/livre opre avec un autre systme, soit /kit/sync. /kit/sync
		contient un rpertoire pour chaque site qui doit tre mis
		 jour. Dans chaque sous-rpertoire de /kit/sync, il y a
		au moins deux fichiers et parfois quelques .zip.
	*/
	int ret = 0;
	/* #Spcification: admin / export / /kit/sync
		Le fichier /kit/sync/XX/version.dat est une copie du fichier
		/kit/ombre/version.dat du site XX. En calculant la diffrence
		entre ce fichier et la version locale de ce fichier
		(/kit/ombre/version.dat), on peut tablir la liste des
		rvisions qui doivent tre transmises au site XX.
		Ces rvisions sont dans /kit/livre et aussi dans les autres
		rpertoires /kit/sync/sites.
	*/
	char path[MAXSIZ_PATH];
	user->maksyncpath (site,path);
	char vpath[MAXSIZ_PATH];
	path_make (path,"version.dat",vpath);
	VERSION_DAT exp (vpath);
	int save_exp = 0;
	int nb=princ.getnb();
	for (int i=0; i<nb; i++){
		VERSION_ONE *one = princ.get(i);
		if (one->getprog()!=NULL){
			int no;
			const REVISION *rev = one->getrev();
			VERSION_ONE *sone = exp.get(rev,no);
			if (sone == NULL){
				// Il faut exporter cette rvision au site
				char revstr[MAXSIZ_REVISION];
				rev->format (revstr);
				char revpath[MAXSIZ_NAME];
				rev->formatpath(revpath);
				char basedir[MAXSIZ_PATH];
				if (export_locate (user,revpath,basedir)){
					/* #Spcification: admin / export / script
						Le fichier /kit/sync/XX/envoie est script UNIX qui est
						activ par le programme prjexpr et transmet chaque rvision
						selon le mcanisme appropri pour un site.

						Il recoit comme seul argument le path absolue (sans
						extension) du repertoire ou du fichier .zip qui
						contient la livraison.
					*/
					char cmd[MAXSIZ_PATH];
					path_make (path,"envoie",cmd);
					path_2os (cmd,cmd);
					strcat (cmd," ");
					path_2os (basedir,basedir);
					strcat (cmd,basedir);
					strcat (cmd," ");
					strcat (cmd,revpath);
					if (prjexpr_system(cmd)==0){
						log.printf ("exporte %s\n",revstr);
						VERSION_ONE *newone = new VERSION_ONE (one);
						if (newone != NULL){
							exp.add (newone);
							save_exp = 1;
						}
					}else{
						log.printf ("**** Erreur durant l'exportation de %s\n"
							,revstr);
						xconf_error (MSG_U(E_ERREXP
							,"Erreur durant l'exportation\n"
							 "de la rvision %s")
							,revstr);
						ret = -1;
					}
				}else{
					log.printf ("**** Pas de livraison disponible pour"
						" exporter %s\n",revstr);
					xconf_error (MSG_U(E_MISSEXP
						,"Pas de livraison disponible\n"
						 "pour exporter la rvision %s")
						,revstr);
					ret = -1;
				}
			}
		}
	}
	if (save_exp) exp.save(user);
	return ret;
}
